from .olvid.daemon.services.v1.admin_service_pb2_grpc import ClientKeyAdminServiceStub
from .olvid.daemon.services.v1.admin_service_pb2_grpc import IdentityAdminServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import IdentityCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import InvitationCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import ContactCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import GroupCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import DiscussionCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import MessageCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import AttachmentCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import StorageCommandServiceStub
from .olvid.daemon.services.v1.command_service_pb2_grpc import DiscussionStorageCommandServiceStub
from .olvid.daemon.services.v1.notification_service_pb2_grpc import InvitationNotificationServiceStub
from .olvid.daemon.services.v1.notification_service_pb2_grpc import ContactNotificationServiceStub
from .olvid.daemon.services.v1.notification_service_pb2_grpc import GroupNotificationServiceStub
from .olvid.daemon.services.v1.notification_service_pb2_grpc import DiscussionNotificationServiceStub
from .olvid.daemon.services.v1.notification_service_pb2_grpc import MessageNotificationServiceStub
from .olvid.daemon.services.v1.notification_service_pb2_grpc import AttachmentNotificationServiceStub



from google.protobuf.json_format import Parse, ParseError

from ..interactive_tree import interactive_tree
from ..tools.cli_tools import *
from ..tools.click_wrappers import WrapperGroup


#####
# storage
#####
@interactive_tree.group("storage", short_help="(advanced) access a client key associated storage", cls=WrapperGroup)
def storage_tree():
	pass


#####
# storage get
#####
# noinspection PyProtectedMember
@storage_tree.command("get")
@click.option("-a", "--all", "get_all", is_flag=True)
@click.option("-d", "discussion_id", nargs=1, type=click.INT, default=0)
@click.argument("keys", nargs=-1, type=click.STRING)
@click.option("--filter", "filter_", type=str)
@click.option("-f", "--fields", "fields", type=str)
async def storage_get(get_all: bool, keys: tuple[str], discussion_id: int, fields: str, filter_: str = ""):
	# build filter
	storage_filter: datatypes.StorageElementFilter = datatypes.StorageElementFilter()
	if filter_:
		try:
			parsed_message = Parse(filter_, datatypes.StorageElementFilter()._to_native(storage_filter))
			storage_filter = datatypes.StorageElementFilter._from_native(parsed_message)
		except ParseError as e:
			print_error_message(f"Cannot parse filter: {e}")
			return

	if get_all:
		# global storage
		elements = [element async for element in ClientSingleton.get_client().storage_list(filter=storage_filter)]
		if elements:
			print_normal_message("Global storage:", "")
		for element in elements:
			filter_fields_and_print_normal_message(element, fields)

		# discussion storage
		async for discussion in ClientSingleton.get_client().discussion_list():
			elements = [element async for element in ClientSingleton.get_client().discussion_storage_list(discussion_id=discussion.id, filter=storage_filter)]
			if elements:
				print_normal_message(f"Discussion id: {discussion.id}", "")
				for element in elements:
					filter_fields_and_print_normal_message(element, fields)

	elif not keys:
		if discussion_id:
			async for element in ClientSingleton.get_client().discussion_storage_list(discussion_id=discussion_id, filter=storage_filter):
				print_normal_message(f"{element.key}: {element.value}", f"{element.key}: {element.value}")
		else:
			async for element in ClientSingleton.get_client().storage_list(filter=storage_filter):
				print_normal_message(f"{element.key}: {element.value}", f"{element.key}: {element.value}")

	# get all storage entries
	else:
		for key in keys:
			if discussion_id:
				value = await ClientSingleton.get_client().discussion_storage_get(key=key, discussion_id=discussion_id)
			else:
				value = await ClientSingleton.get_client().storage_get(key)
			print_normal_message(f"{key}: {value}", f"{value}")


#####
# storage set
#####
@storage_tree.command("set")
@click.argument("key", nargs=1, required=True, type=click.STRING)
@click.option("-d", "discussion_id", nargs=1, type=click.INT, default=0)
@click.argument("value", nargs=1, required=True, type=click.STRING)
async def storage_set(key: str, value: str, discussion_id: int):
	if discussion_id:
		await ClientSingleton.get_client().discussion_storage_set(key=key, value=value, discussion_id=discussion_id)
	else:
		await ClientSingleton.get_client().storage_set(key=key, value=value)
	print_command_result("Element successfully set")


#####
# storage rm
#####
@storage_tree.command("rm")
@click.option("-a", "--all", "delete_all", is_flag=True)
@click.option("-d", "discussion_id", nargs=1, type=click.INT, default=0)
@click.argument("keys", nargs=-1, type=click.STRING)
async def storage_delete(delete_all: bool, discussion_id: int, keys: tuple[str]):
	if delete_all:
		element_deleted = False
		if discussion_id:
			async for element in ClientSingleton.get_client().discussion_storage_list(discussion_id=discussion_id):
				element_deleted = True
				await ClientSingleton.get_client().discussion_storage_unset(key=element.key, discussion_id=discussion_id)
				print_command_result(f"Deleted element: {element.key}={element.value}")
			if not element_deleted:
				print_warning_message("No element to delete")
		else:
			async for element in ClientSingleton.get_client().storage_list():
				element_deleted = True
				await ClientSingleton.get_client().storage_unset(element.key)
				print_command_result(f"Deleted element: {element.key}={element.value}")
			if not element_deleted:
				print_warning_message("No element to delete")
		return

	for key in keys:
		if discussion_id:
			previous_value = await ClientSingleton.get_client().discussion_storage_unset(discussion_id=discussion_id, key=key)
			print_command_result(f"Element deleted: {key}: {previous_value}")
		else:
			previous_value = await ClientSingleton.get_client().storage_unset(key)
			print_command_result(f"Element deleted: {key}: {previous_value}")



import asyncio
import base64
from warnings import warn
import os
from typing import Callable, Optional, Coroutine, Union, Any, AsyncIterator, TYPE_CHECKING

if TYPE_CHECKING:
	from ..core.OlvidClient import OlvidClient

from ..core.logger import core_logger
from .Notifications import NOTIFICATIONS
from ..internal import types


class GenericNotificationListener:
	"""
	GenericNotificationListener a basic notification listener to use within OlvidClient and other subclasses.

	A listener is a method called every time a notification is triggered. When you add a listener to an OlvidClient it will
	automatically register to this notification and handler will be called every time a notification is sent by daemon.

	We do not recommend that you use GenericNotificationListener directly. Instead, you should use one of the provided
	listeners in the ListenersImplementation file.
	You can access them like this:
	```
	from olvid import listeners
	listeners.MessageReceivedListener(handler=lambda m: a)
	```
	Like this you won't need to specify the notification you want to listen to.
	Also, you won't need to use protobuf Notification messages, message are already un-wrapped and handler ill receive notification content.
	For example MessageReceivedListener.handler will receive a datatypes.Message item, not a MessageReceivedNotification
	as a GenericNotificationListener will receive if listening to MessageReceivedNotification.
	"""
	NotificationHandlerType = Callable[[types.NotificationMessageType], Optional[Coroutine]]
	CheckerType = Callable[[types.NotificationMessageType], Union[bool, Coroutine[Any, Any, bool]]]

	def __init__(self, notification_type: NOTIFICATIONS, handler: NotificationHandlerType, checkers: list[CheckerType] = None, count: int = -1, priority: int = 0):
		self._notification_type: NOTIFICATIONS = notification_type
		self._handler = handler
		self._checkers = checkers if checkers else []
		if checkers:
			warn("Listener.checkers will be deprecated in future release", PendingDeprecationWarning, stacklevel=10)
		self._count: int = count
		# we consider that 0 and -1 are endless listeners, -1 for legacy reasons and 0 because it will be normal value in future (notification filtering feature)
		self._endless: bool = count <= 0
		self._priority: int = priority
		self._finished: bool = False

		self._iterator_args: dict = {}

		# we compute listener key after end of init to let child classes fill iterator args
		self._listener_key: Optional[str] = None

	@property
	def notification_type(self) -> NOTIFICATIONS:
		return self._notification_type

	@property
	def priority(self) -> int:
		return self._priority

	@property
	def listener_key(self) -> str:
		if self._listener_key is None:
			self._listener_key = self._generate_listener_key()
		return self._listener_key

	@property
	def count(self) -> int:
		return self._count

	def add_checker(self, checker: CheckerType):
		self._checkers.append(checker)

	@property
	def is_finished(self) -> bool:
		return self._finished

	def mark_as_finished(self):
		self._finished = True

	async def handle_notification(self, notification_message: types.NotificationMessageType, remove_listener_callback: Callable[[
		"GenericNotificationListener"], None]):
		for checker in self._checkers:
			ret = checker(notification_message)
			if asyncio.iscoroutine(ret):
				ret = await ret
			if not ret:
				return

		# check this was not marked as finished before start (not supposed to happen)
		if self._finished:
			return

		try:
			res = self._handler(notification_message)
			if asyncio.iscoroutine(res):
				await res
		except Exception:
			core_logger.exception(f"{self.__class__.__name__}: unexpected exception: {self}")
		finally:
			if not self._endless:
				if self._count > 0:
					self._count -= 1
				if self._count == 0:
					self.mark_as_finished()
					remove_listener_callback(self)

	def _create_iterator(self, client: "OlvidClient") -> AsyncIterator:
		return getattr(client, f"_notif_{self.notification_type.name.lower()}")(**self._iterator_args)

	def _generate_listener_key(self) -> str:
		key: str = f"{self.notification_type.name}"
		# if listener have iterator args (we ignore count arg) we create a unique key (daemon response stream might differ even with same parameters, for example if a count parameter is set)
		if self._iterator_args.get("count") or len(self._iterator_args) > 1:
			for k, v in self._iterator_args.items():
				key += f"_{k}-{v}"
			key += f"_{base64.b64encode(os.urandom(12))}"
		return key

	def __str__(self):
		return f"{self.listener_key}: {self._handler.__name__} (count: {self.count})"

	def __repr__(self):
		return self.__str__()



from .logger import tools_logger
from .AutoInvitationBot import AutoInvitationBot
from .MonitoringBot import MonitoringBot
from .SelfCleaningBot import SelfCleaningBot
from .DiscussionRetentionPolicyBot import DiscussionRetentionPolicyBot
from .DelayedTasksHolder import DelayedTasksHolder
from .ChatBot import ChatBot

del logger, tools_logger



####
# WARNING: DO NOT EDIT: this code is automatically generated, see overlay_generator/generate_protobuf_overlay
####

from __future__ import annotations  # this block is necessary for compilation
from typing import TYPE_CHECKING  # this block is necessary for compilation
if TYPE_CHECKING:  # this block is necessary for compilation
	from ...core.OlvidClient import OlvidClient  # this block is necessary for compilation
import grpc
from typing import AsyncIterator, Coroutine, Any
from ...protobuf import olvid
from ...datatypes import *


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyListRequest:
	def __init__(self, client: OlvidClient = None, filter: "ClientKeyFilter" = None):
		self._client: OlvidClient = client
		self.filter: ClientKeyFilter = filter

	def _update_content(self, client_key_list_request: ClientKeyListRequest) -> None:
		self.filter: ClientKeyFilter = client_key_list_request.filter

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyListRequest":
		return ClientKeyListRequest(client=self._client, filter=self.filter._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListRequest, client: OlvidClient = None) -> "ClientKeyListRequest":
		return ClientKeyListRequest(client, filter=ClientKeyFilter._from_native(native_message.filter, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListRequest], client: OlvidClient = None) -> list["ClientKeyListRequest"]:
		return [ClientKeyListRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListRequest], client: OlvidClient = None) -> "ClientKeyListRequest":
		native_message = await promise
		return ClientKeyListRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyListRequest"]):
		if messages is None:
			return []
		return [ClientKeyListRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyListRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListRequest(filter=ClientKeyFilter._to_native(message.filter if message.filter else None))

	def __str__(self):
		s: str = ''
		if self.filter:
			s += f'filter: ({self.filter}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyListRequest):
			return False
		return self.filter == other.filter

	def __bool__(self):
		return bool(self.filter)

	def __hash__(self):
		return hash(self.filter)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyListRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.filter is None or self.filter._test_assertion(expected.filter)
		except AssertionError as e:
			raise AssertionError("filter: " + str(e))
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyListResponse:
	def __init__(self, client: OlvidClient = None, client_keys: "list[ClientKey]" = None):
		self._client: OlvidClient = client
		self.client_keys: list[ClientKey] = client_keys

	def _update_content(self, client_key_list_response: ClientKeyListResponse) -> None:
		self.client_keys: list[ClientKey] = client_key_list_response.client_keys

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyListResponse":
		return ClientKeyListResponse(client=self._client, client_keys=[e._clone() for e in self.client_keys])

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListResponse, client: OlvidClient = None) -> "ClientKeyListResponse":
		return ClientKeyListResponse(client, client_keys=ClientKey._from_native_list(native_message.client_keys, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListResponse], client: OlvidClient = None) -> list["ClientKeyListResponse"]:
		return [ClientKeyListResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListResponse], client: OlvidClient = None) -> "ClientKeyListResponse":
		native_message = await promise
		return ClientKeyListResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyListResponse"]):
		if messages is None:
			return []
		return [ClientKeyListResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyListResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListResponse(client_keys=ClientKey._to_native_list(message.client_keys if message.client_keys else None))

	def __str__(self):
		s: str = ''
		if self.client_keys:
			s += f'client_keys: {[str(el) for el in self.client_keys]}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyListResponse):
			return False
		return self.client_keys == other.client_keys

	def __bool__(self):
		return bool(self.client_keys)

	def __hash__(self):
		return hash(tuple(self.client_keys))

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyListResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		pass  # print("Warning: test_assertion: skipped a list field client_keys")
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyGetRequest:
	def __init__(self, client: OlvidClient = None, client_key: str = ""):
		self._client: OlvidClient = client
		self.client_key: str = client_key

	def _update_content(self, client_key_get_request: ClientKeyGetRequest) -> None:
		self.client_key: str = client_key_get_request.client_key

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyGetRequest":
		return ClientKeyGetRequest(client=self._client, client_key=self.client_key)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetRequest, client: OlvidClient = None) -> "ClientKeyGetRequest":
		return ClientKeyGetRequest(client, client_key=native_message.client_key)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetRequest], client: OlvidClient = None) -> list["ClientKeyGetRequest"]:
		return [ClientKeyGetRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetRequest], client: OlvidClient = None) -> "ClientKeyGetRequest":
		native_message = await promise
		return ClientKeyGetRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyGetRequest"]):
		if messages is None:
			return []
		return [ClientKeyGetRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyGetRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetRequest(client_key=message.client_key if message.client_key else None)

	def __str__(self):
		s: str = ''
		if self.client_key:
			s += f'client_key: {self.client_key}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyGetRequest):
			return False
		return self.client_key == other.client_key

	def __bool__(self):
		return self.client_key != ""

	def __hash__(self):
		return hash(self.client_key)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyGetRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		assert expected.client_key == "" or self.client_key == expected.client_key, "Invalid value: client_key: " + str(expected.client_key) + " != " + str(self.client_key)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyGetResponse:
	def __init__(self, client: OlvidClient = None, client_key: "ClientKey" = None):
		self._client: OlvidClient = client
		self.client_key: ClientKey = client_key

	def _update_content(self, client_key_get_response: ClientKeyGetResponse) -> None:
		self.client_key: ClientKey = client_key_get_response.client_key

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyGetResponse":
		return ClientKeyGetResponse(client=self._client, client_key=self.client_key._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetResponse, client: OlvidClient = None) -> "ClientKeyGetResponse":
		return ClientKeyGetResponse(client, client_key=ClientKey._from_native(native_message.client_key, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetResponse], client: OlvidClient = None) -> list["ClientKeyGetResponse"]:
		return [ClientKeyGetResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetResponse], client: OlvidClient = None) -> "ClientKeyGetResponse":
		native_message = await promise
		return ClientKeyGetResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyGetResponse"]):
		if messages is None:
			return []
		return [ClientKeyGetResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyGetResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetResponse(client_key=ClientKey._to_native(message.client_key if message.client_key else None))

	def __str__(self):
		s: str = ''
		if self.client_key:
			s += f'client_key: ({self.client_key}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyGetResponse):
			return False
		return self.client_key == other.client_key

	def __bool__(self):
		return bool(self.client_key)

	def __hash__(self):
		return hash(self.client_key)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyGetResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.client_key is None or self.client_key._test_assertion(expected.client_key)
		except AssertionError as e:
			raise AssertionError("client_key: " + str(e))
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyNewRequest:
	def __init__(self, client: OlvidClient = None, name: str = "", identity_id: int = 0):
		self._client: OlvidClient = client
		self.name: str = name
		self.identity_id: int = identity_id

	def _update_content(self, client_key_new_request: ClientKeyNewRequest) -> None:
		self.name: str = client_key_new_request.name
		self.identity_id: int = client_key_new_request.identity_id

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyNewRequest":
		return ClientKeyNewRequest(client=self._client, name=self.name, identity_id=self.identity_id)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewRequest, client: OlvidClient = None) -> "ClientKeyNewRequest":
		return ClientKeyNewRequest(client, name=native_message.name, identity_id=native_message.identity_id)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewRequest], client: OlvidClient = None) -> list["ClientKeyNewRequest"]:
		return [ClientKeyNewRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewRequest], client: OlvidClient = None) -> "ClientKeyNewRequest":
		native_message = await promise
		return ClientKeyNewRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyNewRequest"]):
		if messages is None:
			return []
		return [ClientKeyNewRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyNewRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewRequest(name=message.name if message.name else None, identity_id=message.identity_id if message.identity_id else None)

	def __str__(self):
		s: str = ''
		if self.name:
			s += f'name: {self.name}, '
		if self.identity_id:
			s += f'identity_id: {self.identity_id}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyNewRequest):
			return False
		return self.name == other.name and self.identity_id == other.identity_id

	def __bool__(self):
		return self.name != "" or self.identity_id != 0

	def __hash__(self):
		return hash((self.name, self.identity_id))

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyNewRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		assert expected.name == "" or self.name == expected.name, "Invalid value: name: " + str(expected.name) + " != " + str(self.name)
		assert expected.identity_id == 0 or self.identity_id == expected.identity_id, "Invalid value: identity_id: " + str(expected.identity_id) + " != " + str(self.identity_id)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyNewResponse:
	def __init__(self, client: OlvidClient = None, client_key: "ClientKey" = None):
		self._client: OlvidClient = client
		self.client_key: ClientKey = client_key

	def _update_content(self, client_key_new_response: ClientKeyNewResponse) -> None:
		self.client_key: ClientKey = client_key_new_response.client_key

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyNewResponse":
		return ClientKeyNewResponse(client=self._client, client_key=self.client_key._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewResponse, client: OlvidClient = None) -> "ClientKeyNewResponse":
		return ClientKeyNewResponse(client, client_key=ClientKey._from_native(native_message.client_key, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewResponse], client: OlvidClient = None) -> list["ClientKeyNewResponse"]:
		return [ClientKeyNewResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewResponse], client: OlvidClient = None) -> "ClientKeyNewResponse":
		native_message = await promise
		return ClientKeyNewResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyNewResponse"]):
		if messages is None:
			return []
		return [ClientKeyNewResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyNewResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewResponse(client_key=ClientKey._to_native(message.client_key if message.client_key else None))

	def __str__(self):
		s: str = ''
		if self.client_key:
			s += f'client_key: ({self.client_key}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyNewResponse):
			return False
		return self.client_key == other.client_key

	def __bool__(self):
		return bool(self.client_key)

	def __hash__(self):
		return hash(self.client_key)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyNewResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.client_key is None or self.client_key._test_assertion(expected.client_key)
		except AssertionError as e:
			raise AssertionError("client_key: " + str(e))
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyDeleteRequest:
	def __init__(self, client: OlvidClient = None, client_key: str = ""):
		self._client: OlvidClient = client
		self.client_key: str = client_key

	def _update_content(self, client_key_delete_request: ClientKeyDeleteRequest) -> None:
		self.client_key: str = client_key_delete_request.client_key

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyDeleteRequest":
		return ClientKeyDeleteRequest(client=self._client, client_key=self.client_key)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteRequest, client: OlvidClient = None) -> "ClientKeyDeleteRequest":
		return ClientKeyDeleteRequest(client, client_key=native_message.client_key)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteRequest], client: OlvidClient = None) -> list["ClientKeyDeleteRequest"]:
		return [ClientKeyDeleteRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteRequest], client: OlvidClient = None) -> "ClientKeyDeleteRequest":
		native_message = await promise
		return ClientKeyDeleteRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyDeleteRequest"]):
		if messages is None:
			return []
		return [ClientKeyDeleteRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyDeleteRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteRequest(client_key=message.client_key if message.client_key else None)

	def __str__(self):
		s: str = ''
		if self.client_key:
			s += f'client_key: {self.client_key}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyDeleteRequest):
			return False
		return self.client_key == other.client_key

	def __bool__(self):
		return self.client_key != ""

	def __hash__(self):
		return hash(self.client_key)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyDeleteRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		assert expected.client_key == "" or self.client_key == expected.client_key, "Invalid value: client_key: " + str(expected.client_key) + " != " + str(self.client_key)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class ClientKeyDeleteResponse:
	def __init__(self, client: OlvidClient = None):
		self._client: OlvidClient = client

	def _update_content(self, client_key_delete_response: ClientKeyDeleteResponse) -> None:
		pass

	# noinspection PyProtectedMember
	def _clone(self) -> "ClientKeyDeleteResponse":
		return ClientKeyDeleteResponse(client=self._client)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteResponse, client: OlvidClient = None) -> "ClientKeyDeleteResponse":
		return ClientKeyDeleteResponse(client)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteResponse], client: OlvidClient = None) -> list["ClientKeyDeleteResponse"]:
		return [ClientKeyDeleteResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteResponse], client: OlvidClient = None) -> "ClientKeyDeleteResponse":
		native_message = await promise
		return ClientKeyDeleteResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["ClientKeyDeleteResponse"]):
		if messages is None:
			return []
		return [ClientKeyDeleteResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["ClientKeyDeleteResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteResponse()

	def __str__(self):
		s: str = ''
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, ClientKeyDeleteResponse):
			return False
		return 

	def __bool__(self):
		return False

	def __hash__(self):
		return hash(())

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, ClientKeyDeleteResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)

		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityListRequest:
	def __init__(self, client: OlvidClient = None, filter: "IdentityFilter" = None):
		self._client: OlvidClient = client
		self.filter: IdentityFilter = filter

	def _update_content(self, identity_list_request: IdentityListRequest) -> None:
		self.filter: IdentityFilter = identity_list_request.filter

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityListRequest":
		return IdentityListRequest(client=self._client, filter=self.filter._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityListRequest, client: OlvidClient = None) -> "IdentityListRequest":
		return IdentityListRequest(client, filter=IdentityFilter._from_native(native_message.filter, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityListRequest], client: OlvidClient = None) -> list["IdentityListRequest"]:
		return [IdentityListRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityListRequest], client: OlvidClient = None) -> "IdentityListRequest":
		native_message = await promise
		return IdentityListRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityListRequest"]):
		if messages is None:
			return []
		return [IdentityListRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityListRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityListRequest(filter=IdentityFilter._to_native(message.filter if message.filter else None))

	def __str__(self):
		s: str = ''
		if self.filter:
			s += f'filter: ({self.filter}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityListRequest):
			return False
		return self.filter == other.filter

	def __bool__(self):
		return bool(self.filter)

	def __hash__(self):
		return hash(self.filter)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityListRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.filter is None or self.filter._test_assertion(expected.filter)
		except AssertionError as e:
			raise AssertionError("filter: " + str(e))
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityListResponse:
	def __init__(self, client: OlvidClient = None, identities: "list[Identity]" = None):
		self._client: OlvidClient = client
		self.identities: list[Identity] = identities

	def _update_content(self, identity_list_response: IdentityListResponse) -> None:
		self.identities: list[Identity] = identity_list_response.identities

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityListResponse":
		return IdentityListResponse(client=self._client, identities=[e._clone() for e in self.identities])

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityListResponse, client: OlvidClient = None) -> "IdentityListResponse":
		return IdentityListResponse(client, identities=Identity._from_native_list(native_message.identities, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityListResponse], client: OlvidClient = None) -> list["IdentityListResponse"]:
		return [IdentityListResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityListResponse], client: OlvidClient = None) -> "IdentityListResponse":
		native_message = await promise
		return IdentityListResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityListResponse"]):
		if messages is None:
			return []
		return [IdentityListResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityListResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityListResponse(identities=Identity._to_native_list(message.identities if message.identities else None))

	def __str__(self):
		s: str = ''
		if self.identities:
			s += f'identities: {[str(el) for el in self.identities]}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityListResponse):
			return False
		return self.identities == other.identities

	def __bool__(self):
		return bool(self.identities)

	def __hash__(self):
		return hash(tuple(self.identities))

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityListResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		pass  # print("Warning: test_assertion: skipped a list field identities")
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityAdminGetRequest:
	def __init__(self, client: OlvidClient = None, identity_id: int = 0):
		self._client: OlvidClient = client
		self.identity_id: int = identity_id

	def _update_content(self, identity_admin_get_request: IdentityAdminGetRequest) -> None:
		self.identity_id: int = identity_admin_get_request.identity_id

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityAdminGetRequest":
		return IdentityAdminGetRequest(client=self._client, identity_id=self.identity_id)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetRequest, client: OlvidClient = None) -> "IdentityAdminGetRequest":
		return IdentityAdminGetRequest(client, identity_id=native_message.identity_id)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetRequest], client: OlvidClient = None) -> list["IdentityAdminGetRequest"]:
		return [IdentityAdminGetRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetRequest], client: OlvidClient = None) -> "IdentityAdminGetRequest":
		native_message = await promise
		return IdentityAdminGetRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityAdminGetRequest"]):
		if messages is None:
			return []
		return [IdentityAdminGetRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityAdminGetRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetRequest(identity_id=message.identity_id if message.identity_id else None)

	def __str__(self):
		s: str = ''
		if self.identity_id:
			s += f'identity_id: {self.identity_id}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityAdminGetRequest):
			return False
		return self.identity_id == other.identity_id

	def __bool__(self):
		return self.identity_id != 0

	def __hash__(self):
		return hash(self.identity_id)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityAdminGetRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		assert expected.identity_id == 0 or self.identity_id == expected.identity_id, "Invalid value: identity_id: " + str(expected.identity_id) + " != " + str(self.identity_id)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityAdminGetResponse:
	def __init__(self, client: OlvidClient = None, identity: "Identity" = None):
		self._client: OlvidClient = client
		self.identity: Identity = identity

	def _update_content(self, identity_admin_get_response: IdentityAdminGetResponse) -> None:
		self.identity: Identity = identity_admin_get_response.identity

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityAdminGetResponse":
		return IdentityAdminGetResponse(client=self._client, identity=self.identity._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetResponse, client: OlvidClient = None) -> "IdentityAdminGetResponse":
		return IdentityAdminGetResponse(client, identity=Identity._from_native(native_message.identity, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetResponse], client: OlvidClient = None) -> list["IdentityAdminGetResponse"]:
		return [IdentityAdminGetResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetResponse], client: OlvidClient = None) -> "IdentityAdminGetResponse":
		native_message = await promise
		return IdentityAdminGetResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityAdminGetResponse"]):
		if messages is None:
			return []
		return [IdentityAdminGetResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityAdminGetResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetResponse(identity=Identity._to_native(message.identity if message.identity else None))

	def __str__(self):
		s: str = ''
		if self.identity:
			s += f'identity: ({self.identity}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityAdminGetResponse):
			return False
		return self.identity == other.identity

	def __bool__(self):
		return bool(self.identity)

	def __hash__(self):
		return hash(self.identity)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityAdminGetResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.identity is None or self.identity._test_assertion(expected.identity)
		except AssertionError as e:
			raise AssertionError("identity: " + str(e))
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityNewRequest:
	def __init__(self, client: OlvidClient = None, identity_details: "IdentityDetails" = None, server_url: str = "", api_key: str = ""):
		self._client: OlvidClient = client
		self.identity_details: IdentityDetails = identity_details
		self.server_url: str = server_url
		self.api_key: str = api_key

	def _update_content(self, identity_new_request: IdentityNewRequest) -> None:
		self.identity_details: IdentityDetails = identity_new_request.identity_details
		self.server_url: str = identity_new_request.server_url
		self.api_key: str = identity_new_request.api_key

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityNewRequest":
		return IdentityNewRequest(client=self._client, identity_details=self.identity_details._clone(), server_url=self.server_url, api_key=self.api_key)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewRequest, client: OlvidClient = None) -> "IdentityNewRequest":
		return IdentityNewRequest(client, identity_details=IdentityDetails._from_native(native_message.identity_details, client=client), server_url=native_message.server_url, api_key=native_message.api_key)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewRequest], client: OlvidClient = None) -> list["IdentityNewRequest"]:
		return [IdentityNewRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewRequest], client: OlvidClient = None) -> "IdentityNewRequest":
		native_message = await promise
		return IdentityNewRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityNewRequest"]):
		if messages is None:
			return []
		return [IdentityNewRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityNewRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewRequest(identity_details=IdentityDetails._to_native(message.identity_details if message.identity_details else None), server_url=message.server_url if message.server_url else None, api_key=message.api_key if message.api_key else None)

	def __str__(self):
		s: str = ''
		if self.identity_details:
			s += f'identity_details: ({self.identity_details}), '
		if self.server_url:
			s += f'server_url: {self.server_url}, '
		if self.api_key:
			s += f'api_key: {self.api_key}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityNewRequest):
			return False
		return self.identity_details == other.identity_details and self.server_url == other.server_url and self.api_key == other.api_key

	def __bool__(self):
		return bool(self.identity_details) or self.server_url != "" or self.api_key != ""

	def __hash__(self):
		return hash((self.identity_details, self.server_url, self.api_key))

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityNewRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.identity_details is None or self.identity_details._test_assertion(expected.identity_details)
		except AssertionError as e:
			raise AssertionError("identity_details: " + str(e))
		assert expected.server_url == "" or self.server_url == expected.server_url, "Invalid value: server_url: " + str(expected.server_url) + " != " + str(self.server_url)
		assert expected.api_key == "" or self.api_key == expected.api_key, "Invalid value: api_key: " + str(expected.api_key) + " != " + str(self.api_key)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityNewResponse:
	def __init__(self, client: OlvidClient = None, identity: "Identity" = None):
		self._client: OlvidClient = client
		self.identity: Identity = identity

	def _update_content(self, identity_new_response: IdentityNewResponse) -> None:
		self.identity: Identity = identity_new_response.identity

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityNewResponse":
		return IdentityNewResponse(client=self._client, identity=self.identity._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewResponse, client: OlvidClient = None) -> "IdentityNewResponse":
		return IdentityNewResponse(client, identity=Identity._from_native(native_message.identity, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewResponse], client: OlvidClient = None) -> list["IdentityNewResponse"]:
		return [IdentityNewResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewResponse], client: OlvidClient = None) -> "IdentityNewResponse":
		native_message = await promise
		return IdentityNewResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityNewResponse"]):
		if messages is None:
			return []
		return [IdentityNewResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityNewResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewResponse(identity=Identity._to_native(message.identity if message.identity else None))

	def __str__(self):
		s: str = ''
		if self.identity:
			s += f'identity: ({self.identity}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityNewResponse):
			return False
		return self.identity == other.identity

	def __bool__(self):
		return bool(self.identity)

	def __hash__(self):
		return hash(self.identity)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityNewResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.identity is None or self.identity._test_assertion(expected.identity)
		except AssertionError as e:
			raise AssertionError("identity: " + str(e))
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityDeleteRequest:
	def __init__(self, client: OlvidClient = None, identity_id: int = 0):
		self._client: OlvidClient = client
		self.identity_id: int = identity_id

	def _update_content(self, identity_delete_request: IdentityDeleteRequest) -> None:
		self.identity_id: int = identity_delete_request.identity_id

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityDeleteRequest":
		return IdentityDeleteRequest(client=self._client, identity_id=self.identity_id)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteRequest, client: OlvidClient = None) -> "IdentityDeleteRequest":
		return IdentityDeleteRequest(client, identity_id=native_message.identity_id)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteRequest], client: OlvidClient = None) -> list["IdentityDeleteRequest"]:
		return [IdentityDeleteRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteRequest], client: OlvidClient = None) -> "IdentityDeleteRequest":
		native_message = await promise
		return IdentityDeleteRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityDeleteRequest"]):
		if messages is None:
			return []
		return [IdentityDeleteRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityDeleteRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteRequest(identity_id=message.identity_id if message.identity_id else None)

	def __str__(self):
		s: str = ''
		if self.identity_id:
			s += f'identity_id: {self.identity_id}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityDeleteRequest):
			return False
		return self.identity_id == other.identity_id

	def __bool__(self):
		return self.identity_id != 0

	def __hash__(self):
		return hash(self.identity_id)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityDeleteRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		assert expected.identity_id == 0 or self.identity_id == expected.identity_id, "Invalid value: identity_id: " + str(expected.identity_id) + " != " + str(self.identity_id)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityDeleteResponse:
	def __init__(self, client: OlvidClient = None):
		self._client: OlvidClient = client

	def _update_content(self, identity_delete_response: IdentityDeleteResponse) -> None:
		pass

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityDeleteResponse":
		return IdentityDeleteResponse(client=self._client)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteResponse, client: OlvidClient = None) -> "IdentityDeleteResponse":
		return IdentityDeleteResponse(client)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteResponse], client: OlvidClient = None) -> list["IdentityDeleteResponse"]:
		return [IdentityDeleteResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteResponse], client: OlvidClient = None) -> "IdentityDeleteResponse":
		native_message = await promise
		return IdentityDeleteResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityDeleteResponse"]):
		if messages is None:
			return []
		return [IdentityDeleteResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityDeleteResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteResponse()

	def __str__(self):
		s: str = ''
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityDeleteResponse):
			return False
		return 

	def __bool__(self):
		return False

	def __hash__(self):
		return hash(())

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityDeleteResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)

		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityKeycloakNewRequest:
	def __init__(self, client: OlvidClient = None, configuration_link: str = ""):
		self._client: OlvidClient = client
		self.configuration_link: str = configuration_link

	def _update_content(self, identity_keycloak_new_request: IdentityKeycloakNewRequest) -> None:
		self.configuration_link: str = identity_keycloak_new_request.configuration_link

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityKeycloakNewRequest":
		return IdentityKeycloakNewRequest(client=self._client, configuration_link=self.configuration_link)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewRequest, client: OlvidClient = None) -> "IdentityKeycloakNewRequest":
		return IdentityKeycloakNewRequest(client, configuration_link=native_message.configuration_link)

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewRequest], client: OlvidClient = None) -> list["IdentityKeycloakNewRequest"]:
		return [IdentityKeycloakNewRequest._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewRequest], client: OlvidClient = None) -> "IdentityKeycloakNewRequest":
		native_message = await promise
		return IdentityKeycloakNewRequest._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityKeycloakNewRequest"]):
		if messages is None:
			return []
		return [IdentityKeycloakNewRequest._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityKeycloakNewRequest"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewRequest(configuration_link=message.configuration_link if message.configuration_link else None)

	def __str__(self):
		s: str = ''
		if self.configuration_link:
			s += f'configuration_link: {self.configuration_link}, '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityKeycloakNewRequest):
			return False
		return self.configuration_link == other.configuration_link

	def __bool__(self):
		return self.configuration_link != ""

	def __hash__(self):
		return hash(self.configuration_link)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityKeycloakNewRequest):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		assert expected.configuration_link == "" or self.configuration_link == expected.configuration_link, "Invalid value: configuration_link: " + str(expected.configuration_link) + " != " + str(self.configuration_link)
		return True


# noinspection PyProtectedMember,PyShadowingBuiltins
class IdentityKeycloakNewResponse:
	def __init__(self, client: OlvidClient = None, identity: "Identity" = None):
		self._client: OlvidClient = client
		self.identity: Identity = identity

	def _update_content(self, identity_keycloak_new_response: IdentityKeycloakNewResponse) -> None:
		self.identity: Identity = identity_keycloak_new_response.identity

	# noinspection PyProtectedMember
	def _clone(self) -> "IdentityKeycloakNewResponse":
		return IdentityKeycloakNewResponse(client=self._client, identity=self.identity._clone())

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native(native_message: olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewResponse, client: OlvidClient = None) -> "IdentityKeycloakNewResponse":
		return IdentityKeycloakNewResponse(client, identity=Identity._from_native(native_message.identity, client=client))

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	def _from_native_list(native_message_list: list[olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewResponse], client: OlvidClient = None) -> list["IdentityKeycloakNewResponse"]:
		return [IdentityKeycloakNewResponse._from_native(native_message, client=client) for native_message in native_message_list]

	# noinspection PyUnresolvedReferences,PyUnusedLocal,PyProtectedMember
	@staticmethod
	async def _from_native_promise(promise: Coroutine[Any, Any, olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewResponse], client: OlvidClient = None) -> "IdentityKeycloakNewResponse":
		native_message = await promise
		return IdentityKeycloakNewResponse._from_native(native_message, client=client)

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native_list(messages: list["IdentityKeycloakNewResponse"]):
		if messages is None:
			return []
		return [IdentityKeycloakNewResponse._to_native(message) for message in messages]

	# noinspection PyUnresolvedReferences,PyProtectedMember
	@staticmethod
	def _to_native(message: Optional["IdentityKeycloakNewResponse"]):
		if message is None:
			return None
		return olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewResponse(identity=Identity._to_native(message.identity if message.identity else None))

	def __str__(self):
		s: str = ''
		if self.identity:
			s += f'identity: ({self.identity}), '
		return s.removesuffix(', ')

	def __eq__(self, other):
		if not isinstance(other, IdentityKeycloakNewResponse):
			return False
		return self.identity == other.identity

	def __bool__(self):
		return bool(self.identity)

	def __hash__(self):
		return hash(self.identity)

	# For tests routines
	# noinspection DuplicatedCode,PyProtectedMember
	def _test_assertion(self, expected):
		if not isinstance(expected, IdentityKeycloakNewResponse):
			assert False, "Invalid type: " + str(type(expected).__name__) + " != " + str(type(self).__name__)
		try:
			assert expected.identity is None or self.identity._test_assertion(expected.identity)
		except AssertionError as e:
			raise AssertionError("identity: " + str(e))
		return True


class ClientKeyAdminServiceStub:
	def __init__(self, client: OlvidClient, channel: grpc.Channel):
		self.__stub: olvid.daemon.services.v1.admin_service_pb2_grpc.ClientKeyAdminServiceStub = olvid.daemon.services.v1.admin_service_pb2_grpc.ClientKeyAdminServiceStub(channel=channel)
		self._client: OlvidClient = client

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def client_key_list(self, client_key_list_request: ClientKeyListRequest) -> AsyncIterator[ClientKeyListResponse]:
		# noinspection PyUnresolvedReferences,PyProtectedMember
		async def response_iterator(iterator: AsyncIterator[olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListResponse]) -> AsyncIterator[ClientKeyListResponse]:
			async for native_message in iterator.__aiter__():
				yield ClientKeyListResponse._from_native(native_message, client=self._client)
		overlay_object = client_key_list_request
		return response_iterator(self.__stub.ClientKeyList(olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyListRequest(filter=ClientKeyFilter._to_native(overlay_object.filter)), metadata=self._client.grpc_metadata))

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def client_key_get(self, client_key_get_request: ClientKeyGetRequest) -> Coroutine[Any, Any, ClientKeyGetResponse]:
		overlay_object = client_key_get_request
		return ClientKeyGetResponse._from_native_promise(self.__stub.ClientKeyGet(olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyGetRequest(client_key=overlay_object.client_key), metadata=self._client.grpc_metadata), client=self._client)

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def client_key_new(self, client_key_new_request: ClientKeyNewRequest) -> Coroutine[Any, Any, ClientKeyNewResponse]:
		overlay_object = client_key_new_request
		return ClientKeyNewResponse._from_native_promise(self.__stub.ClientKeyNew(olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyNewRequest(name=overlay_object.name, identity_id=overlay_object.identity_id), metadata=self._client.grpc_metadata), client=self._client)

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def client_key_delete(self, client_key_delete_request: ClientKeyDeleteRequest) -> Coroutine[Any, Any, ClientKeyDeleteResponse]:
		overlay_object = client_key_delete_request
		return ClientKeyDeleteResponse._from_native_promise(self.__stub.ClientKeyDelete(olvid.daemon.admin.v1.client_key_admin_pb2.ClientKeyDeleteRequest(client_key=overlay_object.client_key), metadata=self._client.grpc_metadata), client=self._client)


class IdentityAdminServiceStub:
	def __init__(self, client: OlvidClient, channel: grpc.Channel):
		self.__stub: olvid.daemon.services.v1.admin_service_pb2_grpc.IdentityAdminServiceStub = olvid.daemon.services.v1.admin_service_pb2_grpc.IdentityAdminServiceStub(channel=channel)
		self._client: OlvidClient = client

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def identity_list(self, identity_list_request: IdentityListRequest) -> AsyncIterator[IdentityListResponse]:
		# noinspection PyUnresolvedReferences,PyProtectedMember
		async def response_iterator(iterator: AsyncIterator[olvid.daemon.admin.v1.identity_admin_pb2.IdentityListResponse]) -> AsyncIterator[IdentityListResponse]:
			async for native_message in iterator.__aiter__():
				yield IdentityListResponse._from_native(native_message, client=self._client)
		overlay_object = identity_list_request
		return response_iterator(self.__stub.IdentityList(olvid.daemon.admin.v1.identity_admin_pb2.IdentityListRequest(filter=IdentityFilter._to_native(overlay_object.filter)), metadata=self._client.grpc_metadata))

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def identity_admin_get(self, identity_admin_get_request: IdentityAdminGetRequest) -> Coroutine[Any, Any, IdentityAdminGetResponse]:
		overlay_object = identity_admin_get_request
		return IdentityAdminGetResponse._from_native_promise(self.__stub.IdentityAdminGet(olvid.daemon.admin.v1.identity_admin_pb2.IdentityAdminGetRequest(identity_id=overlay_object.identity_id), metadata=self._client.grpc_metadata), client=self._client)

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def identity_delete(self, identity_delete_request: IdentityDeleteRequest) -> Coroutine[Any, Any, IdentityDeleteResponse]:
		overlay_object = identity_delete_request
		return IdentityDeleteResponse._from_native_promise(self.__stub.IdentityDelete(olvid.daemon.admin.v1.identity_admin_pb2.IdentityDeleteRequest(identity_id=overlay_object.identity_id), metadata=self._client.grpc_metadata), client=self._client)

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def identity_new(self, identity_new_request: IdentityNewRequest) -> Coroutine[Any, Any, IdentityNewResponse]:
		overlay_object = identity_new_request
		return IdentityNewResponse._from_native_promise(self.__stub.IdentityNew(olvid.daemon.admin.v1.identity_admin_pb2.IdentityNewRequest(identity_details=IdentityDetails._to_native(overlay_object.identity_details), server_url=overlay_object.server_url, api_key=overlay_object.api_key), metadata=self._client.grpc_metadata), client=self._client)

	# noinspection PyUnresolvedReferences,PyProtectedMember,PyUnusedLocal
	def identity_keycloak_new(self, identity_keycloak_new_request: IdentityKeycloakNewRequest) -> Coroutine[Any, Any, IdentityKeycloakNewResponse]:
		overlay_object = identity_keycloak_new_request
		return IdentityKeycloakNewResponse._from_native_promise(self.__stub.IdentityKeycloakNew(olvid.daemon.admin.v1.identity_admin_pb2.IdentityKeycloakNewRequest(configuration_link=overlay_object.configuration_link), metadata=self._client.grpc_metadata), client=self._client)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/notification/v1/discussion_notifications.proto
# Protobuf Python Version: 5.27.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    3,
    '',
    'olvid/daemon/notification/v1/discussion_notifications.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....olvid.daemon.datatypes.v1 import discussion_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_discussion__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;olvid/daemon/notification/v1/discussion_notifications.proto\x12\x1colvid.daemon.notification.v1\x1a*olvid/daemon/datatypes/v1/discussion.proto\"&\n$SubscribeToDiscussionNewNotification\"b\n\x19\x44iscussionNewNotification\x12\x45\n\ndiscussion\x18\x01 \x01(\x0b\x32%.olvid.daemon.datatypes.v1.DiscussionR\ndiscussion\")\n\'SubscribeToDiscussionLockedNotification\"e\n\x1c\x44iscussionLockedNotification\x12\x45\n\ndiscussion\x18\x01 \x01(\x0b\x32%.olvid.daemon.datatypes.v1.DiscussionR\ndiscussion\"/\n-SubscribeToDiscussionTitleUpdatedNotification\"\x92\x01\n\"DiscussionTitleUpdatedNotification\x12\x45\n\ndiscussion\x18\x01 \x01(\x0b\x32%.olvid.daemon.datatypes.v1.DiscussionR\ndiscussion\x12%\n\x0eprevious_title\x18\x02 \x01(\tR\rpreviousTitle\"2\n0SubscribeToDiscussionSettingsUpdatedNotification\"\xd5\x01\n%DiscussionSettingsUpdatedNotification\x12P\n\x0cnew_settings\x18\x01 \x01(\x0b\x32-.olvid.daemon.datatypes.v1.DiscussionSettingsR\x0bnewSettings\x12Z\n\x11previous_settings\x18\x02 \x01(\x0b\x32-.olvid.daemon.datatypes.v1.DiscussionSettingsR\x10previousSettingsB\xe4\x01\n com.olvid.daemon.notification.v1B\x1c\x44iscussionNotificationsProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODN\xaa\x02\x1cOlvid.Daemon.Notification.V1\xca\x02\x1cOlvid\\Daemon\\Notification\\V1\xe2\x02(Olvid\\Daemon\\Notification\\V1\\GPBMetadata\xea\x02\x1fOlvid::Daemon::Notification::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.notification.v1.discussion_notifications_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n com.olvid.daemon.notification.v1B\034DiscussionNotificationsProtoP\001Z\017olvid.io/daemon\242\002\003ODN\252\002\034Olvid.Daemon.Notification.V1\312\002\034Olvid\\Daemon\\Notification\\V1\342\002(Olvid\\Daemon\\Notification\\V1\\GPBMetadata\352\002\037Olvid::Daemon::Notification::V1'
  _globals['_SUBSCRIBETODISCUSSIONNEWNOTIFICATION']._serialized_start=137
  _globals['_SUBSCRIBETODISCUSSIONNEWNOTIFICATION']._serialized_end=175
  _globals['_DISCUSSIONNEWNOTIFICATION']._serialized_start=177
  _globals['_DISCUSSIONNEWNOTIFICATION']._serialized_end=275
  _globals['_SUBSCRIBETODISCUSSIONLOCKEDNOTIFICATION']._serialized_start=277
  _globals['_SUBSCRIBETODISCUSSIONLOCKEDNOTIFICATION']._serialized_end=318
  _globals['_DISCUSSIONLOCKEDNOTIFICATION']._serialized_start=320
  _globals['_DISCUSSIONLOCKEDNOTIFICATION']._serialized_end=421
  _globals['_SUBSCRIBETODISCUSSIONTITLEUPDATEDNOTIFICATION']._serialized_start=423
  _globals['_SUBSCRIBETODISCUSSIONTITLEUPDATEDNOTIFICATION']._serialized_end=470
  _globals['_DISCUSSIONTITLEUPDATEDNOTIFICATION']._serialized_start=473
  _globals['_DISCUSSIONTITLEUPDATEDNOTIFICATION']._serialized_end=619
  _globals['_SUBSCRIBETODISCUSSIONSETTINGSUPDATEDNOTIFICATION']._serialized_start=621
  _globals['_SUBSCRIBETODISCUSSIONSETTINGSUPDATEDNOTIFICATION']._serialized_end=671
  _globals['_DISCUSSIONSETTINGSUPDATEDNOTIFICATION']._serialized_start=674
  _globals['_DISCUSSIONSETTINGSUPDATEDNOTIFICATION']._serialized_end=887
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/command/v1/message_commands.proto
# Protobuf Python Version: 5.27.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    3,
    '',
    'olvid/daemon/command/v1/message_commands.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....olvid.daemon.datatypes.v1 import attachment_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_attachment__pb2
from .....olvid.daemon.datatypes.v1 import message_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.olvid/daemon/command/v1/message_commands.proto\x12\x17olvid.daemon.command.v1\x1a*olvid/daemon/datatypes/v1/attachment.proto\x1a\'olvid/daemon/datatypes/v1/message.proto\"\x8e\x01\n\x12MessageListRequest\x12\x45\n\x06\x66ilter\x18\x01 \x01(\x0b\x32(.olvid.daemon.datatypes.v1.MessageFilterH\x00R\x06\x66ilter\x88\x01\x01\x12\x1b\n\x06unread\x18\x02 \x01(\x08H\x01R\x06unread\x88\x01\x01\x42\t\n\x07_filterB\t\n\x07_unread\"U\n\x13MessageListResponse\x12>\n\x08messages\x18\x01 \x03(\x0b\x32\".olvid.daemon.datatypes.v1.MessageR\x08messages\"X\n\x11MessageGetRequest\x12\x43\n\nmessage_id\x18\x01 \x01(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdR\tmessageId\"R\n\x12MessageGetResponse\x12<\n\x07message\x18\x01 \x01(\x0b\x32\".olvid.daemon.datatypes.v1.MessageR\x07message\"\x17\n\x15MessageRefreshRequest\"\x18\n\x16MessageRefreshResponse\"\xa3\x01\n\x14MessageDeleteRequest\x12\x43\n\nmessage_id\x18\x01 \x01(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdR\tmessageId\x12\x30\n\x11\x64\x65lete_everywhere\x18\x02 \x01(\x08H\x00R\x10\x64\x65leteEverywhere\x88\x01\x01\x42\x14\n\x12_delete_everywhere\"\x17\n\x15MessageDeleteResponse\"\xda\x02\n\x12MessageSendRequest\x12#\n\rdiscussion_id\x18\x01 \x01(\x04R\x0c\x64iscussionId\x12\x12\n\x04\x62ody\x18\x02 \x01(\tR\x04\x62ody\x12\x44\n\x08reply_id\x18\x03 \x01(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdH\x00R\x07replyId\x88\x01\x01\x12W\n\x0c\x65phemerality\x18\x04 \x01(\x0b\x32..olvid.daemon.datatypes.v1.MessageEphemeralityH\x01R\x0c\x65phemerality\x88\x01\x01\x12\x35\n\x14\x64isable_link_preview\x18\x05 \x01(\x08H\x02R\x12\x64isableLinkPreview\x88\x01\x01\x42\x0b\n\t_reply_idB\x0f\n\r_ephemeralityB\x17\n\x15_disable_link_preview\"S\n\x13MessageSendResponse\x12<\n\x07message\x18\x01 \x01(\x0b\x32\".olvid.daemon.datatypes.v1.MessageR\x07message\"\xd5\x01\n!MessageSendWithAttachmentsRequest\x12`\n\x08metadata\x18\x01 \x01(\x0b\x32\x42.olvid.daemon.command.v1.MessageSendWithAttachmentsRequestMetadataH\x00R\x08metadata\x12\x1a\n\x07payload\x18\x02 \x01(\x0cH\x00R\x07payload\x12\'\n\x0e\x66ile_delimiter\x18\x03 \x01(\x08H\x00R\rfileDelimiterB\t\n\x07request\"\x9f\x04\n)MessageSendWithAttachmentsRequestMetadata\x12#\n\rdiscussion_id\x18\x01 \x01(\x04R\x0c\x64iscussionId\x12\x17\n\x04\x62ody\x18\x02 \x01(\tH\x00R\x04\x62ody\x88\x01\x01\x12\x44\n\x08reply_id\x18\x03 \x01(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdH\x01R\x07replyId\x88\x01\x01\x12W\n\x0c\x65phemerality\x18\x04 \x01(\x0b\x32..olvid.daemon.datatypes.v1.MessageEphemeralityH\x02R\x0c\x65phemerality\x88\x01\x01\x12\x35\n\x14\x64isable_link_preview\x18\x05 \x01(\x08H\x03R\x12\x64isableLinkPreview\x88\x01\x01\x12]\n\x05\x66iles\x18\n \x03(\x0b\x32G.olvid.daemon.command.v1.MessageSendWithAttachmentsRequestMetadata.FileR\x05\x66iles\x1a?\n\x04\x46ile\x12\x1a\n\x08\x66ilename\x18\x01 \x01(\tR\x08\x66ilename\x12\x1b\n\tfile_size\x18\x02 \x01(\x04R\x08\x66ileSizeB\x07\n\x05_bodyB\x0b\n\t_reply_idB\x0f\n\r_ephemeralityB\x17\n\x15_disable_link_preview\"\xab\x01\n\"MessageSendWithAttachmentsResponse\x12<\n\x07message\x18\x01 \x01(\x0b\x32\".olvid.daemon.datatypes.v1.MessageR\x07message\x12G\n\x0b\x61ttachments\x18\x02 \x03(\x0b\x32%.olvid.daemon.datatypes.v1.AttachmentR\x0b\x61ttachments\"v\n\x13MessageReactRequest\x12\x43\n\nmessage_id\x18\x01 \x01(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdR\tmessageId\x12\x1a\n\x08reaction\x18\x02 \x01(\tR\x08reaction\"\x16\n\x14MessageReactResponse\"\x82\x01\n\x18MessageUpdateBodyRequest\x12\x43\n\nmessage_id\x18\x01 \x01(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdR\tmessageId\x12!\n\x0cupdated_body\x18\x02 \x01(\tR\x0bupdatedBody\"\x1b\n\x19MessageUpdateBodyResponse\"=\n\x16MessageSendVoipRequest\x12#\n\rdiscussion_id\x18\x01 \x01(\x04R\x0c\x64iscussionId\"\x19\n\x17MessageSendVoipResponseB\xc3\x01\n\x1b\x63om.olvid.daemon.command.v1B\x14MessageCommandsProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODC\xaa\x02\x17Olvid.Daemon.Command.V1\xca\x02\x17Olvid\\Daemon\\Command\\V1\xe2\x02#Olvid\\Daemon\\Command\\V1\\GPBMetadata\xea\x02\x1aOlvid::Daemon::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.command.v1.message_commands_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033com.olvid.daemon.command.v1B\024MessageCommandsProtoP\001Z\017olvid.io/daemon\242\002\003ODC\252\002\027Olvid.Daemon.Command.V1\312\002\027Olvid\\Daemon\\Command\\V1\342\002#Olvid\\Daemon\\Command\\V1\\GPBMetadata\352\002\032Olvid::Daemon::Command::V1'
  _globals['_MESSAGELISTREQUEST']._serialized_start=161
  _globals['_MESSAGELISTREQUEST']._serialized_end=303
  _globals['_MESSAGELISTRESPONSE']._serialized_start=305
  _globals['_MESSAGELISTRESPONSE']._serialized_end=390
  _globals['_MESSAGEGETREQUEST']._serialized_start=392
  _globals['_MESSAGEGETREQUEST']._serialized_end=480
  _globals['_MESSAGEGETRESPONSE']._serialized_start=482
  _globals['_MESSAGEGETRESPONSE']._serialized_end=564
  _globals['_MESSAGEREFRESHREQUEST']._serialized_start=566
  _globals['_MESSAGEREFRESHREQUEST']._serialized_end=589
  _globals['_MESSAGEREFRESHRESPONSE']._serialized_start=591
  _globals['_MESSAGEREFRESHRESPONSE']._serialized_end=615
  _globals['_MESSAGEDELETEREQUEST']._serialized_start=618
  _globals['_MESSAGEDELETEREQUEST']._serialized_end=781
  _globals['_MESSAGEDELETERESPONSE']._serialized_start=783
  _globals['_MESSAGEDELETERESPONSE']._serialized_end=806
  _globals['_MESSAGESENDREQUEST']._serialized_start=809
  _globals['_MESSAGESENDREQUEST']._serialized_end=1155
  _globals['_MESSAGESENDRESPONSE']._serialized_start=1157
  _globals['_MESSAGESENDRESPONSE']._serialized_end=1240
  _globals['_MESSAGESENDWITHATTACHMENTSREQUEST']._serialized_start=1243
  _globals['_MESSAGESENDWITHATTACHMENTSREQUEST']._serialized_end=1456
  _globals['_MESSAGESENDWITHATTACHMENTSREQUESTMETADATA']._serialized_start=1459
  _globals['_MESSAGESENDWITHATTACHMENTSREQUESTMETADATA']._serialized_end=2002
  _globals['_MESSAGESENDWITHATTACHMENTSREQUESTMETADATA_FILE']._serialized_start=1875
  _globals['_MESSAGESENDWITHATTACHMENTSREQUESTMETADATA_FILE']._serialized_end=1938
  _globals['_MESSAGESENDWITHATTACHMENTSRESPONSE']._serialized_start=2005
  _globals['_MESSAGESENDWITHATTACHMENTSRESPONSE']._serialized_end=2176
  _globals['_MESSAGEREACTREQUEST']._serialized_start=2178
  _globals['_MESSAGEREACTREQUEST']._serialized_end=2296
  _globals['_MESSAGEREACTRESPONSE']._serialized_start=2298
  _globals['_MESSAGEREACTRESPONSE']._serialized_end=2320
  _globals['_MESSAGEUPDATEBODYREQUEST']._serialized_start=2323
  _globals['_MESSAGEUPDATEBODYREQUEST']._serialized_end=2453
  _globals['_MESSAGEUPDATEBODYRESPONSE']._serialized_start=2455
  _globals['_MESSAGEUPDATEBODYRESPONSE']._serialized_end=2482
  _globals['_MESSAGESENDVOIPREQUEST']._serialized_start=2484
  _globals['_MESSAGESENDVOIPREQUEST']._serialized_end=2545
  _globals['_MESSAGESENDVOIPRESPONSE']._serialized_start=2547
  _globals['_MESSAGESENDVOIPRESPONSE']._serialized_end=2572
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/notification/v1/contact_notifications.proto
# Protobuf Python Version: 5.27.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    3,
    '',
    'olvid/daemon/notification/v1/contact_notifications.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....olvid.daemon.datatypes.v1 import contact_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_contact__pb2
from .....olvid.daemon.datatypes.v1 import identity_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_identity__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8olvid/daemon/notification/v1/contact_notifications.proto\x12\x1colvid.daemon.notification.v1\x1a\'olvid/daemon/datatypes/v1/contact.proto\x1a(olvid/daemon/datatypes/v1/identity.proto\"#\n!SubscribeToContactNewNotification\"V\n\x16\x43ontactNewNotification\x12<\n\x07\x63ontact\x18\x01 \x01(\x0b\x32\".olvid.daemon.datatypes.v1.ContactR\x07\x63ontact\"\'\n%SubscribeToContactDeletedNotification\"Z\n\x1a\x43ontactDeletedNotification\x12<\n\x07\x63ontact\x18\x01 \x01(\x0b\x32\".olvid.daemon.datatypes.v1.ContactR\x07\x63ontact\".\n,SubscribeToContactDetailsUpdatedNotification\"\xb8\x01\n!ContactDetailsUpdatedNotification\x12<\n\x07\x63ontact\x18\x01 \x01(\x0b\x32\".olvid.daemon.datatypes.v1.ContactR\x07\x63ontact\x12U\n\x10previous_details\x18\x02 \x01(\x0b\x32*.olvid.daemon.datatypes.v1.IdentityDetailsR\x0fpreviousDetailsB\xe1\x01\n com.olvid.daemon.notification.v1B\x19\x43ontactNotificationsProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODN\xaa\x02\x1cOlvid.Daemon.Notification.V1\xca\x02\x1cOlvid\\Daemon\\Notification\\V1\xe2\x02(Olvid\\Daemon\\Notification\\V1\\GPBMetadata\xea\x02\x1fOlvid::Daemon::Notification::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.notification.v1.contact_notifications_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n com.olvid.daemon.notification.v1B\031ContactNotificationsProtoP\001Z\017olvid.io/daemon\242\002\003ODN\252\002\034Olvid.Daemon.Notification.V1\312\002\034Olvid\\Daemon\\Notification\\V1\342\002(Olvid\\Daemon\\Notification\\V1\\GPBMetadata\352\002\037Olvid::Daemon::Notification::V1'
  _globals['_SUBSCRIBETOCONTACTNEWNOTIFICATION']._serialized_start=173
  _globals['_SUBSCRIBETOCONTACTNEWNOTIFICATION']._serialized_end=208
  _globals['_CONTACTNEWNOTIFICATION']._serialized_start=210
  _globals['_CONTACTNEWNOTIFICATION']._serialized_end=296
  _globals['_SUBSCRIBETOCONTACTDELETEDNOTIFICATION']._serialized_start=298
  _globals['_SUBSCRIBETOCONTACTDELETEDNOTIFICATION']._serialized_end=337
  _globals['_CONTACTDELETEDNOTIFICATION']._serialized_start=339
  _globals['_CONTACTDELETEDNOTIFICATION']._serialized_end=429
  _globals['_SUBSCRIBETOCONTACTDETAILSUPDATEDNOTIFICATION']._serialized_start=431
  _globals['_SUBSCRIBETOCONTACTDETAILSUPDATEDNOTIFICATION']._serialized_end=477
  _globals['_CONTACTDETAILSUPDATEDNOTIFICATION']._serialized_start=480
  _globals['_CONTACTDETAILSUPDATEDNOTIFICATION']._serialized_end=664
# @@protoc_insertion_point(module_scope)



import asyncclick as click

from olvid import OlvidClient, OlvidAdminClient, datatypes


class ClientSingleton():
	__client: OlvidClient = None
	__is_current_client_admin: bool = True
	__script_mode: bool = False
	__current_identity_id: int = 0
	__default_admin_client: OlvidAdminClient = None

	@staticmethod
	async def init(identity_id: int = 0):
		ClientSingleton.__current_identity_id = identity_id
		ClientSingleton.__default_admin_client = OlvidAdminClient(identity_id=identity_id)
		ClientSingleton.__client = ClientSingleton.__default_admin_client

	@staticmethod
	async def auto_select_identity():
		if ClientSingleton.is_client_admin():
			async for identity in ClientSingleton.get_client().admin_identity_list():
				ClientSingleton.set_current_identity_id(identity_id=identity.id)
				click.secho("{}".format(f"Currently using identity: {identity.id}"), fg="green")
				return
		else:
			click.secho("ClientSingleton: cannot auto select identity for a non admin client", fg="red", err=True)

	####
	# change current client api
	####
	@staticmethod
	async def impersonate_client_key(client_key: datatypes.ClientKey):
		if client_key.identity_id == 0:
			ClientSingleton.__is_current_client_admin = True
			ClientSingleton.__client = OlvidAdminClient(client_key=client_key.key, identity_id=ClientSingleton.__default_admin_client.current_identity_id)
		else:
			ClientSingleton.__is_current_client_admin = False
			if not ClientSingleton.is_default_client():
				await ClientSingleton.__client.stop()
			ClientSingleton.__client = OlvidClient(client_key=client_key.key)
			ClientSingleton.__current_identity_id = client_key.identity_id

	@staticmethod
	async def use_default_client():
		ClientSingleton.__client = ClientSingleton.__default_admin_client
		ClientSingleton.__is_current_client_admin = True
		ClientSingleton.__current_identity_id = ClientSingleton.__default_admin_client.current_identity_id

	####
	# Access current client and properties
	####
	@staticmethod
	def is_client_admin() -> bool:
		return ClientSingleton.__is_current_client_admin

	@staticmethod
	def is_default_client() -> bool:
		return ClientSingleton.__client == ClientSingleton.__default_admin_client

	@staticmethod
	def get_client() -> OlvidAdminClient:
		if not ClientSingleton.__client:
			raise Exception("ClientSingleton was not initialized")
		return ClientSingleton.__client

	@staticmethod
	def get_current_identity_id():
		return ClientSingleton.__current_identity_id

	@staticmethod
	def set_current_identity_id(identity_id: int):
		if ClientSingleton.is_client_admin():
			client: OlvidAdminClient = ClientSingleton.__client
			client.current_identity_id = identity_id
			ClientSingleton.__current_identity_id = identity_id
		else:
			click.secho("Cannot change current identity for non admin clients", fg="red", err=True)

	####
	# Script mode property
	####
	@staticmethod
	def is_script_mode_enabled():
		return ClientSingleton.__script_mode

	@staticmethod
	def enable_script_mode():
		ClientSingleton.__script_mode = True



import asyncio
from typing import Optional

import readline
import asyncclick as click
import grpc.aio

from .ClientSingleton import ClientSingleton
from ...core.OlvidAdminClient import OlvidAdminClient
from ...datatypes import datatypes
from ...listeners import ListenersImplementation as listeners
from .cli_tools import print_warning_message

def interactive_command(func):
	def wrapper(*args, **kwargs):
		if ClientSingleton.is_script_mode_enabled():
			print_warning_message("WARNING: launching interactive command in script mode")
		try:
			# disable history in interactive mode
			readline.set_auto_history(False)
			return func(*args, **kwargs)
		finally:
			# re-enable history in interactive mode
			readline.set_auto_history(True)
	return wrapper


# raise AbortException
@interactive_command
def ask_question_with_context(question: str, prompt: str = None, fg_color: str = None, bg_color: str = None) -> bool:
	if prompt:
		prompt += " > "
	if prompt and (fg_color or bg_color):
		prompt = click.style(prompt, fg=fg_color, bg=bg_color)
	return click.prompt(prompt + question, type=bool, prompt_suffix=" (y/N)\n>")


@interactive_command
def prompt_with_context(question: str, prompt: str = None, fg_color: str = None, bg_color: str = None) -> str:
	if prompt:
		prompt += " > "
	if prompt and (fg_color or bg_color):
		prompt = click.style(prompt, fg=fg_color, bg=bg_color)
	return click.prompt(prompt + question, type=str, prompt_suffix="\n>")


def print_with_context(text: str, prompt: str = None, fg_color: str = None, bg_color: str = None):
	if prompt:
		prompt += " > "
	if prompt and (fg_color or bg_color):
		prompt = click.style(prompt, fg=fg_color, bg=bg_color)
	print(prompt + text)


# return new discussion
async def contact_new(identity_id: int, prompt: str = None, fg_color: str = None, bg_color: str = None) -> Optional[datatypes.Discussion]:
	identity: datatypes.Identity = await ClientSingleton.get_client().admin_identity_admin_get(identity_id=identity_id)
	print_with_context(f"Send an invitation to this invitation link: {identity.invitation_url}", prompt=prompt, fg_color=fg_color, bg_color=bg_color)

	# create invitation received listener
	client: OlvidAdminClient = OlvidAdminClient(identity_id=identity_id)
	invitations: list[datatypes.Invitation] = []
	client.add_listener(listeners.InvitationReceivedListener(handler=lambda i: invitations.append(i), count=1, checkers=[lambda i: i.status == datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_TO_ACCEPT]))

	# create discussion new listener
	discussion_new_client = OlvidAdminClient(identity_id=identity_id)
	discussions: list[datatypes.Discussion] = []
	discussion_new_client.add_listener(listeners.DiscussionNewListener(handler=lambda i: discussions.append(i), count=1))

	# wait for invitation to arrive
	await client.wait_for_listeners_end()
	invitation: datatypes.Invitation = invitations[0]
	await client.invitation_accept(invitation_id=invitation.id)
	invitation = await invitation.wait_for(listener_class=listeners.InvitationUpdatedListener, extra_checker=lambda i, s: i.status == datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_FOR_SAS_EXCHANGE)

	# enter other device sas
	while True:
		sas_code: str = prompt_with_context("Please enter sas code displayed on the other device", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
		try:
			await client.invitation_sas(invitation_id=invitation.id, sas=sas_code)
		except grpc.aio.AioRpcError:
			print_with_context("Invalid sas code", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
			continue

		await asyncio.sleep(0.1)
		invitation = await client.invitation_get(invitation_id=invitation.id)
		if invitation.status == datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_FOR_SAS_EXCHANGE:
			prompt_with_context("Invalid sas code", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
		else:
			break

	# show your sas and wait for invitation deletion
	print_with_context(f"Please enter this sas code on the other device: {invitation.sas}", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
	await invitation.wait_for(listeners.InvitationDeletedListener)

	await discussion_new_client.stop()
	if discussions:
		return discussions[0]
	else:
		return None


async def invitation_new(identity_id: int, invitation: datatypes.Invitation, prompt: str = None, fg_color: str = None, bg_color: str = None) -> Optional[datatypes.Discussion]:
	client: OlvidAdminClient = OlvidAdminClient(identity_id=identity_id)

	# create discussion new listener
	discussion_new_client = OlvidAdminClient(identity_id=identity_id)
	discussions: list[datatypes.Discussion] = []
	discussion_new_client.add_listener(listeners.DiscussionNewListener(handler=lambda i: discussions.append(i), count=1))

	# wait for other identity to accept
	print_with_context("Waiting for other device to accept invitation ...", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
	invitation = await invitation.wait_for(listener_class=listeners.InvitationUpdatedListener, extra_checker=lambda i, s: i.status == datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_FOR_SAS_EXCHANGE)

	# set other identity sas
	while True:
		sas_code: str = prompt_with_context("Please enter sas code displayed on the other device", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
		try:
			await client.invitation_sas(invitation_id=invitation.id, sas=sas_code)
		except grpc.aio.AioRpcError:
			print_with_context("Invalid sas code", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
			continue

		await asyncio.sleep(0.1)
		invitation = await client.invitation_get(invitation_id=invitation.id)
		if invitation.status == datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_FOR_SAS_EXCHANGE:
			prompt_with_context("Invalid sas code", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
		else:
			break

	# show your sas and wait for invitation deletion
	print_with_context(f"Please enter this sas code on the other device: {invitation.sas}", prompt=prompt, fg_color=fg_color, bg_color=bg_color)
	await invitation.wait_for(listeners.InvitationDeletedListener)

	await discussion_new_client.stop()
	if discussions:
		return discussions[0]
	else:
		return None
